;;;; (net http client) -- client library for HTTP.

;;  Copyright 2001, Evan Prodromou <evan@prodromou.san-francisco.ca.us>.

;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.

;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.

;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(define-module (net http client)
  :use-module (net http message)
  :use-module (net http util)
  :use-module (oop goops)
  :use-module (oop goops describe)
  :use-module (ice-9 format))

(export <http-client> post get head)

;; Compatibility kludge.

(or (defined? 'read-line)
    (use-modules (ice-9 rdelim)))

;;; The version of this module

(define *module-version* "0.3.1")

;;; The User-Agent header we send as a default

(define *user-agent* (string-append "(net http client)/" *module-version*))

(define-class <http-client> (<object>)
  (agent-id #:accessor agent-id #:init-value *user-agent* #:init-keyword #:agent-id))

(define-method (post (client <http-client>) uri params)
  (let* ((parts (parse-uri uri))
	 (request (make <http-request> #:method "POST" #:server (list-ref parts 1)
			#:port (list-ref parts 2) #:uri (list-ref parts 3)
			#:headers (list (cons "Connection" "close")
                                        (cons "User-Agent" (agent-id client))
                                        (cons "Content-Type" "application/x-www-form-urlencoded")
                                        (cons "Host" (list-ref parts 1))))))
    (let ((contentstr (make-content-str params)))
      (header-set! request "Content-Length" (string-length contentstr))
      (call-with-input-string
       contentstr
       (lambda (port)
         (set! (content request) port)
         (let* ((response (send-request request))
                (content (content response)))
           (if (has-header response "Content-Length")
               (let ((count (string->number (header response "Content-Length"))))
                 (copy-port-count content (current-output-port) count))
               (copy-port content (current-output-port)))
           (close content)
           response))))))

(define-method (head (client <http-client>) uri)
  (let* ((parts (parse-uri uri))
	 (request (make <http-request> #:method "HEAD" #:server (list-ref parts 1)
			#:port (list-ref parts 2) #:uri (list-ref parts 3)
			#:headers (list (cons "Connection" "close")
                                        (cons "User-Agent" (agent-id client))
                                        (cons "Host" (list-ref parts 1)))))
	 (response (send-request request)))
    (close (content response))
    response))

(define-method (get (client <http-client>) uri)
  (let* ((parts (parse-uri uri))
	 (request (make <http-request> #:method "GET" #:server (list-ref parts 1)
			#:port (list-ref parts 2) #:uri (list-ref parts 3)
			#:headers (list (cons "Connection" "close")
                                        (cons "User-Agent" (agent-id client))
                                        (cons "Host" (list-ref parts 1)))))
	 (response (send-request request))
	 (content (content response)))
    (if (has-header response "Content-Length")
	(let ((count (string->number (header response "Content-Length"))))
	  (copy-port-count content (current-output-port) count))
        (copy-port content (current-output-port)))
    (close content)
    response))

;; Add post params to a request.

(define (make-content-str params)
  (if (null? params)
      ""
      (let* ((name (caar params))
	     (value (cdar params))
	     (rest (cdr params))
	     (this-arg (format #f "~A=~A" (urlencode name) (urlencode value))))
	(if (null? rest)
	    this-arg
	    (string-append this-arg "&" (make-content-str rest))))))

